@@ -1,8 +1,8 @@
 grts <- function(design, DesignID="Site", SiteBegin=1, type.frame="finite",
    src.frame="shapefile", in.shape=NULL, sp.object=NULL, att.frame=NULL,
    id=NULL, xcoord=NULL, ycoord=NULL, stratum=NULL, mdcaty=NULL, startlev=NULL,
-   maxlev=11, maxtry=1000, shift.grid=TRUE, do.sample=TRUE, shapefile=TRUE,
-   prjfilename=NULL, out.shape="sample") {
+   maxlev=11, maxtry=1000, shift.grid=TRUE, do.sample=rep(TRUE, length(design)),
+   shapefile=TRUE, prjfilename=NULL, out.shape="sample") {
 
 ################################################################################
 # Function: grts
@@ -10,7 +10,7 @@ grts <- function(design, DesignID="Site", SiteBegin=1, type.frame="finite",
 # Programmers: Tony Olsen, Tom Kincaid, Don Stevens, Christian Platt,
 #              Denis White, Richard Remington
 # Date: October 8, 2002
-# Last Revised: February 15, 2007
+# Last Revised: April 23, 2008
 # Description:
 #   This function select a GRTS sample of a finite, linear, or area resource.
 #   Frame elements must be located in 1- or 2-dimensional coordinate system.
@@ -59,13 +59,13 @@ grts <- function(design, DesignID="Site", SiteBegin=1, type.frame="finite",
 #     is included in att.frame.  The default is "shapefile".
 #   in.shape = name (without any extension) of the input shapefile.  If
 #     src.frame equal "shapefile" and in.shape equals NULL, then the shapefile
-#     or shapefiles in the currrent directory are used.  The default is NULL.
+#     or shapefiles in the working directory are used.  The default is NULL.
 #   sp.object = name of the sp package object when src.frame equals "sp.object".
 #     The default is NULL.
 #   att.frame = a data frame composed of attributes associated with elements in
 #     the frame, which must contain the columns used for stratum and mdcaty (if
 #     required).  If src.frame equals "shapefile" and att.frame equals NULL,
-#     then att.frame is created from the dbf file(s) in the current directory.
+#     then att.frame is created from the dbf file(s) in the working directory.
 #     If src.frame equals "att.frame", then att.frame includes columns that
 #     contain x-coordinates and y-coordinates for each element in the frame.
 #     The default is NULL.
@@ -98,10 +98,13 @@ grts <- function(design, DesignID="Site", SiteBegin=1, type.frame="finite",
 #     means shift the grid and FALSE means do not shift the grid, which is
 #     useful if one desires strict spatial stratification by hierarchical grid
 #     cells.  The default is TRUE.
-#   do.sample = option to select a sample, where TRUE means select a sample and
-#     FALSE means return the entire sample frame in reverse hierarchical order.
-#     Note that FALSE can only be used when type.frame equals "points" and
-#     seltype equals "Equal".  The default is TRUE.
+#   do.sample = named vector that provides the option controlling sample
+#     selection for each stratum, where TRUE means select a sample from a
+#     stratum and FALSE means return the sample frame for a stratum in reverse
+#     hierarchical order.  Note that FALSE can only be used when type.frame
+#     equals "points" and seltype equals "Equal".  Names for the vector must
+#     match the names in design.  If the vector is not named, then the names in
+#     design are used.  The default is TRUE for each stratum.
 #   shapefile = option to create a shapefile containing the survey design
 #     information,  where TRUE equals create a shapefile and FALSE equals do
 #     not create a shapefile.  The default is TRUE.
@@ -312,7 +315,24 @@ if(type.frame == "finite") {
          ycoord <- "y"
       temp <- match(c(xcoord, ycoord), names(att.frame), nomatch=0)
       if(any(temp == 0))
-         stop(paste("\nThe names for the columns containing the x-coordinates and y-coordinates,\n\"", xcoord, "\" and \"", ycoord, "\", do not occur among the column names in att.frame.\n", sep=""))
+         stop(paste("\nThe names for one or both of the columns containing the x-coordinates and \ny-coordinates, \"", xcoord, "\" and \"", ycoord, "\", \ndo not occur among the column names in att.frame.", sep=""))
+   }
+
+# Ensure that do.sample is the correct length and is named
+
+   if(length(do.sample) > 1) {
+      if(length(do.sample) != length(design))
+         stop("\nArgument do.sample must be the same length as the design list.")
+      if(is.null(names(do.sample))) {
+         names(do.sample) <- strata.names
+      } else {
+         temp <- match(names(do.sample), strata.names, nomatch=0)
+         if(any(temp) == 0)
+            temp.str <- vecprint(names(do.sample)[temp == 0])
+            stop(paste("\nThe following names in do.sample do not occur among the names in design:\n", temp.str, sep=""))
+      }
+   } else if(is.null(names(do.sample))) {
+      names(do.sample) <- strata.names
    }
 
 # Begin the loop for strata
@@ -362,7 +382,7 @@ if(type.frame == "finite") {
       }
 
 # If seltype is "Unequal", ensure that caty.n is provided and that the names
-# in caty.n and the levels of mdcaty are equivalent
+# in caty.n are included amont the levels of mdcaty
 
       if(design[[s]]$seltype == "Unequal") {
          if(is.null(design[[s]]$caty.n))
@@ -373,12 +393,6 @@ if(type.frame == "finite") {
             temp.str <- vecprint(names(design[[s]]$caty.n)[temp == 0])
             stop(paste("\nThe following names in caty.n for stratum \"", s, "\" do not occur \namong the levels of the mdcaty variable in att.frame:\n", temp.str, sep=""))
          }
-         temp <- match(levels(as.factor(sframe$mdcaty)),
-             names(design[[s]]$caty.n), nomatch=0)
-         if(any(temp == 0)) {
-            temp.str <- vecprint(levels(as.factor(sframe$mdcaty))[temp == 0])
-            stop(paste("\nThe following levels of the mdcaty variable in att.frame do not occur \namong the names in caty.n for stratum \"", s, "\":\n", temp.str, "\nNote that this problem can result both from level(s) in mdcaty that are not \nincluded among the names in caty.n and from typographical errors among the \nvalues in mdcaty.\n", sep=""))
-         }
       }
 
 # Ensure that panel and caty.n contain valid values
@@ -399,6 +413,16 @@ if(type.frame == "finite") {
             stop(paste(" The design list does not not contain any valid values of the caty.n \nargument for stratum \"", s, "\".\n", sep=""))
       }
 
+# As necessary, remove rows from sframe that have values of mdcaty which are not
+# included among the names in caty.n
+
+      if(design[[s]]$seltype == "Unequal") {
+         temp <- sframe$mdcaty %in% names(design[[s]]$caty.n)
+         if(any(!temp)) {
+            sframe <- sframe[temp,]
+         }
+      }
+
 # Determine overall sample size for the stratum
 
       if(is.null(design[[s]]$over))
@@ -434,7 +458,7 @@ if(type.frame == "finite") {
 
       if(grtspts.ind) {
          stmp <- grtspts(src.frame, in.shape, sframe, sum(n.desired), SiteBegin,
-            shift.grid, do.sample, startlev, maxlev)
+            shift.grid, do.sample[s], startlev, maxlev)
       } else {
          stmp <- data.frame(siteID=SiteBegin, id=sframe$id, xcoord=sframe$x,
             ycoord=sframe$y, mdcaty=sframe$mdcaty, wgt=1/sframe$mdm)
@@ -523,13 +547,13 @@ if(type.frame == "finite") {
    SiteBegin <- SiteBegin
 
 # Ensure that att.frame includes a variable named length_mdm that provides the
-# length for each record of the shapefile(s) in the current directory and create
+# length for each record of the shapefile(s) in the working directory and create
 # the variable when necessary
 
    if(is.null(att.frame$length_mdm)) {
       temp <- .Call("getRecordShapeSizes", in.shape)
       if(length(temp) != nrow(att.frame))
-         stop("\nThe number of rows in the attribute data frame does not equal the number of \nrecords in the shapefile(s) in the current directory.")
+         stop("\nThe number of rows in the attribute data frame does not equal the number of \nrecords in the shapefile(s) in the working directory.")
       att.frame$length_mdm <- temp
    }
    elmsize <- "length_mdm"
@@ -589,12 +613,6 @@ if(type.frame == "finite") {
             temp.str <- vecprint(names(design[[s]]$caty.n)[temp == 0])
             stop(paste("\nThe following names in caty.n for stratum \"", s, "\" do not occur \namong the levels of the mdcaty variable in att.frame:\n", temp.str, sep=""))
          }
-         temp <- match(levels(as.factor(sframe$mdcaty)),
-             names(design[[s]]$caty.n), nomatch=0)
-         if(any(temp == 0)) {
-            temp.str <- vecprint(levels(as.factor(sframe$mdcaty))[temp == 0])
-            stop(paste("\nThe following levels of the mdcaty variable in att.frame do not occur \namong the names in caty.n for stratum \"", s, "\":\n", temp.str, "\nNote that this problem can result both from level(s) in mdcaty that are not \nincluded among the names in caty.n and from typographical errors among the \nvalues in mdcaty.\n", sep=""))
-         }
       }
 
 # Ensure that panel and caty.n contain valid values
@@ -615,6 +633,16 @@ if(type.frame == "finite") {
             stop(paste(" The design list does not not contain any valid values of the caty.n \nargument for stratum \"", s, "\".\n", sep=""))
       }
 
+# As necessary, remove rows from sframe that have values of mdcaty which are not
+# included among the names in caty.n
+
+      if(design[[s]]$seltype == "Unequal") {
+         temp <- sframe$mdcaty %in% names(design[[s]]$caty.n)
+         if(any(!temp)) {
+            sframe <- sframe[temp,]
+         }
+      }
+
 # Determine overall sample size for the stratum
 
       if(is.null(design[[s]]$over))
@@ -701,13 +729,13 @@ if(type.frame == "finite") {
    SiteBegin <- SiteBegin
 
 # Ensure that att.frame includes a variable named area_mdm that provides the
-# area for each record of the shapefile(s) in the current directory and create
+# area for each record of the shapefile(s) in the working directory and create
 # the variable when necessary
 
    if(is.null(att.frame$area_mdm)) {
       temp <- .Call("getRecordShapeSizes", in.shape)
       if(length(temp) != nrow(att.frame))
-         stop("\nThe number of rows in the attribute data frame does not equal the number of \nrecords in the shapefile(s) in the current directory.")
+         stop("\nThe number of rows in the attribute data frame does not equal the number of \nrecords in the shapefile(s) in the working directory.")
       att.frame$area_mdm <- temp
    }
    elmsize <- "area_mdm"
@@ -767,12 +795,6 @@ if(type.frame == "finite") {
             temp.str <- vecprint(names(design[[s]]$caty.n)[temp == 0])
             stop(paste("\nThe following names in caty.n for stratum \"", s, "\" do not occur \namong the levels of the mdcaty variable in att.frame:\n", temp.str, sep=""))
          }
-         temp <- match(levels(as.factor(sframe$mdcaty)),
-             names(design[[s]]$caty.n), nomatch=0)
-         if(any(temp == 0)) {
-            temp.str <- vecprint(levels(as.factor(sframe$mdcaty))[temp == 0])
-            stop(paste("\nThe following levels of the mdcaty variable in att.frame do not occur \namong the names in caty.n for stratum \"", s, "\":\n", temp.str, "\nNote that this problem can result both from level(s) in mdcaty that are not \nincluded among the names in caty.n and from typographical errors among the \nvalues in mdcaty.\n", sep=""))
-         }
       }
 
 # Ensure that panel and caty.n contain valid values
@@ -793,6 +815,16 @@ if(type.frame == "finite") {
             stop(paste(" The design list does not not contain any valid values of the caty.n \nargument for stratum \"", s, "\".\n", sep=""))
       }
 
+# As necessary, remove rows from sframe that have values of mdcaty which are not
+# included among the names in caty.n
+
+      if(design[[s]]$seltype == "Unequal") {
+         temp <- sframe$mdcaty %in% names(design[[s]]$caty.n)
+         if(any(!temp)) {
+            sframe <- sframe[temp,]
+         }
+      }
+
 # Determine overall sample size for the stratum
 
       if(is.null(design[[s]]$over))
@@ -994,8 +1026,13 @@ if(shapefile == TRUE) {
    temp <- sapply(sites, is.factor)
    if(any(temp)) {
       sites.tmp <- sites
-      for(i in seq(ncol(sites.tmp))[temp])
+      for(i in seq(ncol(sites.tmp))[temp]) {
          sites.tmp[,i] <- as.character(sites.tmp[,i])
+         temp <- sites.tmp[,i] == "" & !is.na(sites.tmp[,i])
+         if(any(temp)) {
+            sites.tmp[temp,i] <- " "
+         }
+      }
       .Call("writeShapeFilePoint", sites.tmp$xcoord, sites.tmp$ycoord,
          prjfilename, names(sites.tmp), sites.tmp, out.shape)
    } else {
